# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .page import (
    PageResource,
    AsyncPageResource,
    PageResourceWithRawResponse,
    AsyncPageResourceWithRawResponse,
    PageResourceWithStreamingResponse,
    AsyncPageResourceWithStreamingResponse,
)
from .rules import (
    RulesResource,
    AsyncRulesResource,
    RulesResourceWithRawResponse,
    AsyncRulesResourceWithRawResponse,
    RulesResourceWithStreamingResponse,
    AsyncRulesResourceWithStreamingResponse,
)
from .events import (
    EventsResource,
    AsyncEventsResource,
    EventsResourceWithRawResponse,
    AsyncEventsResourceWithRawResponse,
    EventsResourceWithStreamingResponse,
    AsyncEventsResourceWithStreamingResponse,
)
from .settings import (
    SettingsResource,
    AsyncSettingsResource,
    SettingsResourceWithRawResponse,
    AsyncSettingsResourceWithRawResponse,
    SettingsResourceWithStreamingResponse,
    AsyncSettingsResourceWithStreamingResponse,
)
from .statuses import (
    StatusesResource,
    AsyncStatusesResource,
    StatusesResourceWithRawResponse,
    AsyncStatusesResourceWithRawResponse,
    StatusesResourceWithStreamingResponse,
    AsyncStatusesResourceWithStreamingResponse,
)
from .waiting_rooms import (
    WaitingRoomsResource,
    AsyncWaitingRoomsResource,
    WaitingRoomsResourceWithRawResponse,
    AsyncWaitingRoomsResourceWithRawResponse,
    WaitingRoomsResourceWithStreamingResponse,
    AsyncWaitingRoomsResourceWithStreamingResponse,
)

__all__ = [
    "PageResource",
    "AsyncPageResource",
    "PageResourceWithRawResponse",
    "AsyncPageResourceWithRawResponse",
    "PageResourceWithStreamingResponse",
    "AsyncPageResourceWithStreamingResponse",
    "EventsResource",
    "AsyncEventsResource",
    "EventsResourceWithRawResponse",
    "AsyncEventsResourceWithRawResponse",
    "EventsResourceWithStreamingResponse",
    "AsyncEventsResourceWithStreamingResponse",
    "RulesResource",
    "AsyncRulesResource",
    "RulesResourceWithRawResponse",
    "AsyncRulesResourceWithRawResponse",
    "RulesResourceWithStreamingResponse",
    "AsyncRulesResourceWithStreamingResponse",
    "StatusesResource",
    "AsyncStatusesResource",
    "StatusesResourceWithRawResponse",
    "AsyncStatusesResourceWithRawResponse",
    "StatusesResourceWithStreamingResponse",
    "AsyncStatusesResourceWithStreamingResponse",
    "SettingsResource",
    "AsyncSettingsResource",
    "SettingsResourceWithRawResponse",
    "AsyncSettingsResourceWithRawResponse",
    "SettingsResourceWithStreamingResponse",
    "AsyncSettingsResourceWithStreamingResponse",
    "WaitingRoomsResource",
    "AsyncWaitingRoomsResource",
    "WaitingRoomsResourceWithRawResponse",
    "AsyncWaitingRoomsResourceWithRawResponse",
    "WaitingRoomsResourceWithStreamingResponse",
    "AsyncWaitingRoomsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["EntityGetResponse", "IP"]


class IP(BaseModel):
    asn: str

    asn_location: str = FieldInfo(alias="asnLocation")

    asn_name: str = FieldInfo(alias="asnName")

    asn_org_name: str = FieldInfo(alias="asnOrgName")

    ip: str

    ip_version: str = FieldInfo(alias="ipVersion")

    location: str

    location_name: str = FieldInfo(alias="locationName")


class EntityGetResponse(BaseModel):
    ip: IP



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from ...._models import BaseModel

__all__ = ["Route"]


class Route(BaseModel):
    id: Optional[str] = None
    """UUID of the route."""

    comment: Optional[str] = None
    """Optional remark describing the route."""

    created_at: Optional[datetime] = None
    """Timestamp of when the resource was created."""

    deleted_at: Optional[datetime] = None
    """Timestamp of when the resource was deleted.

    If `null`, the resource has not been deleted.
    """

    network: Optional[str] = None
    """The private IPv4 or IPv6 range connected by the route, in CIDR notation."""

    tunnel_id: Optional[str] = None
    """UUID of the tunnel."""

    virtual_network_id: Optional[str] = None
    """UUID of the virtual network."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict
from typing_extensions import Required, TypedDict

__all__ = ["ButtonTextTranslationParam"]


class ButtonTextTranslationParam(TypedDict, total=False):
    accept_all: Required[Dict[str, str]]
    """Object where keys are language codes."""

    confirm_my_choices: Required[Dict[str, str]]
    """Object where keys are language codes."""

    reject_all: Required[Dict[str, str]]
    """Object where keys are language codes."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, cast

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ....pagination import SyncSinglePage, AsyncSinglePage
from ...._base_client import AsyncPaginator, make_request_options
from ....types.zero_trust.networks import (
    virtual_network_edit_params,
    virtual_network_list_params,
    virtual_network_create_params,
)
from ....types.zero_trust.networks.virtual_network import VirtualNetwork

__all__ = ["VirtualNetworksResource", "AsyncVirtualNetworksResource"]


class VirtualNetworksResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> VirtualNetworksResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return VirtualNetworksResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> VirtualNetworksResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return VirtualNetworksResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        name: str,
        comment: str | Omit = omit,
        is_default: bool | Omit = omit,
        is_default_network: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VirtualNetwork:
        """
        Adds a new virtual network to an account.

        Args:
          account_id: Cloudflare account ID

          name: A user-friendly name for the virtual network.

          comment: Optional remark describing the virtual network.

          is_default: If `true`, this virtual network is the default for the account.

          is_default_network: If `true`, this virtual network is the default for the account.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/teamnet/virtual_networks", account_id=account_id),
            body=maybe_transform(
                {
                    "name": name,
                    "comment": comment,
                    "is_default": is_default,
                    "is_default_network": is_default_network,
                },
                virtual_network_create_params.VirtualNetworkCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[VirtualNetwork]._unwrapper,
            ),
            cast_to=cast(Type[VirtualNetwork], ResultWrapper[VirtualNetwork]),
        )

    def list(
        self,
        *,
        account_id: str,
        id: str | Omit = omit,
        is_default: bool | Omit = omit,
        is_default_network: bool | Omit = omit,
        is_deleted: bool | Omit = omit,
        name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[VirtualNetwork]:
        """
        Lists and filters virtual networks in an account.

        Args:
          account_id: Cloudflare account ID

          id: UUID of the virtual network.

          is_default: If `true`, only include the default virtual network. If `false`, exclude the
              default virtual network. If empty, all virtual networks will be included.

          is_default_network: If `true`, only include the default virtual network. If `false`, exclude the
              default virtual network. If empty, all virtual networks will be included.

          is_deleted: If `true`, only include deleted virtual networks. If `false`, exclude deleted
              virtual networks. If empty, all virtual networks will be included.

          name: A user-friendly name for the virtual network.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/teamnet/virtual_networks", account_id=account_id),
            page=SyncSinglePage[VirtualNetwork],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "id": id,
                        "is_default": is_default,
                        "is_default_network": is_default_network,
                        "is_deleted": is_deleted,
                        "name": name,
                    },
                    virtual_network_list_params.VirtualNetworkListParams,
                ),
            ),
            model=VirtualNetwork,
        )

    def delete(
        self,
        virtual_network_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VirtualNetwork:
        """
        Deletes an existing virtual network.

        Args:
          account_id: Cloudflare account ID

          virtual_network_id: UUID of the virtual network.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not virtual_network_id:
            raise ValueError(f"Expected a non-empty value for `virtual_network_id` but received {virtual_network_id!r}")
        return self._delete(
            path_template(
                "/accounts/{account_id}/teamnet/virtual_networks/{virtual_network_id}",
                account_id=account_id,
                virtual_network_id=virtual_network_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[VirtualNetwork]._unwrapper,
            ),
            cast_to=cast(Type[VirtualNetwork], ResultWrapper[VirtualNetwork]),
        )

    def edit(
        self,
        virtual_network_id: str,
        *,
        account_id: str,
        comment: str | Omit = omit,
        is_default_network: bool | Omit = omit,
        name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VirtualNetwork:
        """
        Updates an existing virtual network.

        Args:
          account_id: Cloudflare account ID

          virtual_network_id: UUID of the virtual network.

          comment: Optional remark describing the virtual network.

          is_default_network: If `true`, this virtual network is the default for the account.

          name: A user-friendly name for the virtual network.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not virtual_network_id:
            raise ValueError(f"Expected a non-empty value for `virtual_network_id` but received {virtual_network_id!r}")
        return self._patch(
            path_template(
                "/accounts/{account_id}/teamnet/virtual_networks/{virtual_network_id}",
                account_id=account_id,
                virtual_network_id=virtual_network_id,
            ),
            body=maybe_transform(
                {
                    "comment": comment,
                    "is_default_network": is_default_network,
                    "name": name,
                },
                virtual_network_edit_params.VirtualNetworkEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[VirtualNetwork]._unwrapper,
            ),
            cast_to=cast(Type[VirtualNetwork], ResultWrapper[VirtualNetwork]),
        )

    def get(
        self,
        virtual_network_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VirtualNetwork:
        """
        Get a virtual network.

        Args:
          account_id: Cloudflare account ID

          virtual_network_id: UUID of the virtual network.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not virtual_network_id:
            raise ValueError(f"Expected a non-empty value for `virtual_network_id` but received {virtual_network_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/teamnet/virtual_networks/{virtual_network_id}",
                account_id=account_id,
                virtual_network_id=virtual_network_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[VirtualNetwork]._unwrapper,
            ),
            cast_to=cast(Type[VirtualNetwork], ResultWrapper[VirtualNetwork]),
        )


class AsyncVirtualNetworksResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncVirtualNetworksResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncVirtualNetworksResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncVirtualNetworksResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncVirtualNetworksResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        name: str,
        comment: str | Omit = omit,
        is_default: bool | Omit = omit,
        is_default_network: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VirtualNetwork:
        """
        Adds a new virtual network to an account.

        Args:
          account_id: Cloudflare account ID

          name: A user-friendly name for the virtual network.

          comment: Optional remark describing the virtual network.

          is_default: If `true`, this virtual network is the default for the account.

          is_default_network: If `true`, this virtual network is the default for the account.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/teamnet/virtual_networks", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "name": name,
                    "comment": comment,
                    "is_default": is_default,
                    "is_default_network": is_default_network,
                },
                virtual_network_create_params.VirtualNetworkCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[VirtualNetwork]._unwrapper,
            ),
            cast_to=cast(Type[VirtualNetwork], ResultWrapper[VirtualNetwork]),
        )

    def list(
        self,
        *,
        account_id: str,
        id: str | Omit = omit,
        is_default: bool | Omit = omit,
        is_default_network: bool | Omit = omit,
        is_deleted: bool | Omit = omit,
        name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[VirtualNetwork, AsyncSinglePage[VirtualNetwork]]:
        """
        Lists and filters virtual networks in an account.

        Args:
          account_id: Cloudflare account ID

          id: UUID of the virtual network.

          is_default: If `true`, only include the default virtual network. If `false`, exclude the
              default virtual network. If empty, all virtual networks will be included.

          is_default_network: If `true`, only include the default virtual network. If `false`, exclude the
              default virtual network. If empty, all virtual networks will be included.

          is_deleted: If `true`, only include deleted virtual networks. If `false`, exclude deleted
              virtual networks. If empty, all virtual networks will be included.

          name: A user-friendly name for the virtual network.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/teamnet/virtual_networks", account_id=account_id),
            page=AsyncSinglePage[VirtualNetwork],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "id": id,
                        "is_default": is_default,
                        "is_default_network": is_default_network,
                        "is_deleted": is_deleted,
                        "name": name,
                    },
                    virtual_network_list_params.VirtualNetworkListParams,
                ),
            ),
            model=VirtualNetwork,
        )

    async def delete(
        self,
        virtual_network_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VirtualNetwork:
        """
        Deletes an existing virtual network.

        Args:
          account_id: Cloudflare account ID

          virtual_network_id: UUID of the virtual network.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not virtual_network_id:
            raise ValueError(f"Expected a non-empty value for `virtual_network_id` but received {virtual_network_id!r}")
        return await self._delete(
            path_template(
                "/accounts/{account_id}/teamnet/virtual_networks/{virtual_network_id}",
                account_id=account_id,
                virtual_network_id=virtual_network_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[VirtualNetwork]._unwrapper,
            ),
            cast_to=cast(Type[VirtualNetwork], ResultWrapper[VirtualNetwork]),
        )

    async def edit(
        self,
        virtual_network_id: str,
        *,
        account_id: str,
        comment: str | Omit = omit,
        is_default_network: bool | Omit = omit,
        name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VirtualNetwork:
        """
        Updates an existing virtual network.

        Args:
          account_id: Cloudflare account ID

          virtual_network_id: UUID of the virtual network.

          comment: Optional remark describing the virtual network.

          is_default_network: If `true`, this virtual network is the default for the account.

          name: A user-friendly name for the virtual network.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not virtual_network_id:
            raise ValueError(f"Expected a non-empty value for `virtual_network_id` but received {virtual_network_id!r}")
        return await self._patch(
            path_template(
                "/accounts/{account_id}/teamnet/virtual_networks/{virtual_network_id}",
                account_id=account_id,
                virtual_network_id=virtual_network_id,
            ),
            body=await async_maybe_transform(
                {
                    "comment": comment,
                    "is_default_network": is_default_network,
                    "name": name,
                },
                virtual_network_edit_params.VirtualNetworkEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[VirtualNetwork]._unwrapper,
            ),
            cast_to=cast(Type[VirtualNetwork], ResultWrapper[VirtualNetwork]),
        )

    async def get(
        self,
        virtual_network_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VirtualNetwork:
        """
        Get a virtual network.

        Args:
          account_id: Cloudflare account ID

          virtual_network_id: UUID of the virtual network.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not virtual_network_id:
            raise ValueError(f"Expected a non-empty value for `virtual_network_id` but received {virtual_network_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/teamnet/virtual_networks/{virtual_network_id}",
                account_id=account_id,
                virtual_network_id=virtual_network_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[VirtualNetwork]._unwrapper,
            ),
            cast_to=cast(Type[VirtualNetwork], ResultWrapper[VirtualNetwork]),
        )


class VirtualNetworksResourceWithRawResponse:
    def __init__(self, virtual_networks: VirtualNetworksResource) -> None:
        self._virtual_networks = virtual_networks

        self.create = to_raw_response_wrapper(
            virtual_networks.create,
        )
        self.list = to_raw_response_wrapper(
            virtual_networks.list,
        )
        self.delete = to_raw_response_wrapper(
            virtual_networks.delete,
        )
        self.edit = to_raw_response_wrapper(
            virtual_networks.edit,
        )
        self.get = to_raw_response_wrapper(
            virtual_networks.get,
        )


class AsyncVirtualNetworksResourceWithRawResponse:
    def __init__(self, virtual_networks: AsyncVirtualNetworksResource) -> None:
        self._virtual_networks = virtual_networks

        self.create = async_to_raw_response_wrapper(
            virtual_networks.create,
        )
        self.list = async_to_raw_response_wrapper(
            virtual_networks.list,
        )
        self.delete = async_to_raw_response_wrapper(
            virtual_networks.delete,
        )
        self.edit = async_to_raw_response_wrapper(
            virtual_networks.edit,
        )
        self.get = async_to_raw_response_wrapper(
            virtual_networks.get,
        )


class VirtualNetworksResourceWithStreamingResponse:
    def __init__(self, virtual_networks: VirtualNetworksResource) -> None:
        self._virtual_networks = virtual_networks

        self.create = to_streamed_response_wrapper(
            virtual_networks.create,
        )
        self.list = to_streamed_response_wrapper(
            virtual_networks.list,
        )
        self.delete = to_streamed_response_wrapper(
            virtual_networks.delete,
        )
        self.edit = to_streamed_response_wrapper(
            virtual_networks.edit,
        )
        self.get = to_streamed_response_wrapper(
            virtual_networks.get,
        )


class AsyncVirtualNetworksResourceWithStreamingResponse:
    def __init__(self, virtual_networks: AsyncVirtualNetworksResource) -> None:
        self._virtual_networks = virtual_networks

        self.create = async_to_streamed_response_wrapper(
            virtual_networks.create,
        )
        self.list = async_to_streamed_response_wrapper(
            virtual_networks.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            virtual_networks.delete,
        )
        self.edit = async_to_streamed_response_wrapper(
            virtual_networks.edit,
        )
        self.get = async_to_streamed_response_wrapper(
            virtual_networks.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .dispatch import (
    DispatchResource,
    AsyncDispatchResource,
    DispatchResourceWithRawResponse,
    AsyncDispatchResourceWithRawResponse,
    DispatchResourceWithStreamingResponse,
    AsyncDispatchResourceWithStreamingResponse,
)
from .workers_for_platforms import (
    WorkersForPlatformsResource,
    AsyncWorkersForPlatformsResource,
    WorkersForPlatformsResourceWithRawResponse,
    AsyncWorkersForPlatformsResourceWithRawResponse,
    WorkersForPlatformsResourceWithStreamingResponse,
    AsyncWorkersForPlatformsResourceWithStreamingResponse,
)

__all__ = [
    "DispatchResource",
    "AsyncDispatchResource",
    "DispatchResourceWithRawResponse",
    "AsyncDispatchResourceWithRawResponse",
    "DispatchResourceWithStreamingResponse",
    "AsyncDispatchResourceWithStreamingResponse",
    "WorkersForPlatformsResource",
    "AsyncWorkersForPlatformsResource",
    "WorkersForPlatformsResourceWithRawResponse",
    "AsyncWorkersForPlatformsResourceWithRawResponse",
    "WorkersForPlatformsResourceWithStreamingResponse",
    "AsyncWorkersForPlatformsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, cast
from typing_extensions import Literal

import httpx

from ...._types import Body, Query, Headers, NotGiven, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ...._base_client import make_request_options
from ....types.ai_gateway.billing import spending_limit_create_params
from ....types.ai_gateway.billing.spending_limit_get_response import SpendingLimitGetResponse

__all__ = ["SpendingLimitResource", "AsyncSpendingLimitResource"]


class SpendingLimitResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> SpendingLimitResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return SpendingLimitResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SpendingLimitResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return SpendingLimitResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        amount: int,
        duration: Literal["daily", "weekly", "monthly"],
        strategy: Literal["fixed", "sliding"],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Configure a spending limit with amount, strategy, and duration.

        Args:
          amount: Spending limit amount in cents (min 100).

          duration: Spending limit duration.

          strategy: Spending limit strategy.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/ai-gateway/billing/spending-limit", account_id=account_id),
            body=maybe_transform(
                {
                    "amount": amount,
                    "duration": duration,
                    "strategy": strategy,
                },
                spending_limit_create_params.SpendingLimitCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[object]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )

    def delete(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Remove the spending limit for the account.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._delete(
            path_template("/accounts/{account_id}/ai-gateway/billing/spending-limit", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[object]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )

    def get(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SpendingLimitGetResponse:
        """
        Retrieve the current spending limit configuration for the account.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get(
            path_template("/accounts/{account_id}/ai-gateway/billing/spending-limit", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[SpendingLimitGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[SpendingLimitGetResponse], ResultWrapper[SpendingLimitGetResponse]),
        )


class AsyncSpendingLimitResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncSpendingLimitResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSpendingLimitResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSpendingLimitResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncSpendingLimitResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        amount: int,
        duration: Literal["daily", "weekly", "monthly"],
        strategy: Literal["fixed", "sliding"],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Configure a spending limit with amount, strategy, and duration.

        Args:
          amount: Spending limit amount in cents (min 100).

          duration: Spending limit duration.

          strategy: Spending limit strategy.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/ai-gateway/billing/spending-limit", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "amount": amount,
                    "duration": duration,
                    "strategy": strategy,
                },
                spending_limit_create_params.SpendingLimitCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[object]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )

    async def delete(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Remove the spending limit for the account.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._delete(
            path_template("/accounts/{account_id}/ai-gateway/billing/spending-limit", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[object]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )

    async def get(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SpendingLimitGetResponse:
        """
        Retrieve the current spending limit configuration for the account.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/ai-gateway/billing/spending-limit", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[SpendingLimitGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[SpendingLimitGetResponse], ResultWrapper[SpendingLimitGetResponse]),
        )


class SpendingLimitResourceWithRawResponse:
    def __init__(self, spending_limit: SpendingLimitResource) -> None:
        self._spending_limit = spending_limit

        self.create = to_raw_response_wrapper(
            spending_limit.create,
        )
        self.delete = to_raw_response_wrapper(
            spending_limit.delete,
        )
        self.get = to_raw_response_wrapper(
            spending_limit.get,
        )


class AsyncSpendingLimitResourceWithRawResponse:
    def __init__(self, spending_limit: AsyncSpendingLimitResource) -> None:
        self._spending_limit = spending_limit

        self.create = async_to_raw_response_wrapper(
            spending_limit.create,
        )
        self.delete = async_to_raw_response_wrapper(
            spending_limit.delete,
        )
        self.get = async_to_raw_response_wrapper(
            spending_limit.get,
        )


class SpendingLimitResourceWithStreamingResponse:
    def __init__(self, spending_limit: SpendingLimitResource) -> None:
        self._spending_limit = spending_limit

        self.create = to_streamed_response_wrapper(
            spending_limit.create,
        )
        self.delete = to_streamed_response_wrapper(
            spending_limit.delete,
        )
        self.get = to_streamed_response_wrapper(
            spending_limit.get,
        )


class AsyncSpendingLimitResourceWithStreamingResponse:
    def __init__(self, spending_limit: AsyncSpendingLimitResource) -> None:
        self._spending_limit = spending_limit

        self.create = async_to_streamed_response_wrapper(
            spending_limit.create,
        )
        self.delete = async_to_streamed_response_wrapper(
            spending_limit.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            spending_limit.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ..._models import BaseModel

__all__ = ["PipelineCreateV1Response"]


class PipelineCreateV1Response(BaseModel):
    id: str
    """Indicates a unique identifier for this pipeline."""

    created_at: str

    modified_at: str

    name: str
    """Indicates the name of the Pipeline."""

    sql: str
    """Specifies SQL for the Pipeline processing flow."""

    status: str
    """Indicates the current status of the Pipeline."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

from .tunnel_param import TunnelParam
from ..custom_hostnames.bundle_method import BundleMethod

__all__ = ["KeylessCertificateCreateParams"]


class KeylessCertificateCreateParams(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier."""

    certificate: Required[str]
    """The zone's SSL certificate or SSL certificate and intermediate(s)."""

    host: Required[str]
    """The keyless SSL name."""

    port: Required[float]
    """
    The keyless SSL port used to communicate between Cloudflare and the client's
    Keyless SSL server.
    """

    bundle_method: BundleMethod
    """
    A ubiquitous bundle has the highest probability of being verified everywhere,
    even by clients using outdated or unusual trust stores. An optimal bundle uses
    the shortest chain and newest intermediates. And the force bundle verifies the
    chain, but does not otherwise modify it.
    """

    name: str
    """The keyless SSL name."""

    tunnel: TunnelParam
    """Configuration for using Keyless SSL through a Cloudflare Tunnel"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast
from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ...types.dns import dnssec_edit_params
from ..._base_client import make_request_options
from ...types.dns.dnssec import DNSSEC
from ...types.dns.dnssec_delete_response import DNSSECDeleteResponse

__all__ = ["DNSSECResource", "AsyncDNSSECResource"]


class DNSSECResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> DNSSECResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return DNSSECResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DNSSECResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return DNSSECResourceWithStreamingResponse(self)

    def delete(
        self,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> str:
        """
        Delete DNSSEC.

        Args:
          zone_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._delete(
            path_template("/zones/{zone_id}/dnssec", zone_id=zone_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[DNSSECDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(Type[str], ResultWrapper[str]),
        )

    def edit(
        self,
        *,
        zone_id: str,
        dnssec_multi_signer: bool | Omit = omit,
        dnssec_presigned: bool | Omit = omit,
        dnssec_use_nsec3: bool | Omit = omit,
        status: Literal["active", "disabled"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[DNSSEC]:
        """
        Enable or disable DNSSEC.

        Args:
          zone_id: Identifier.

          dnssec_multi_signer: If true, multi-signer DNSSEC is enabled on the zone, allowing multiple providers
              to serve a DNSSEC-signed zone at the same time. This is required for DNSKEY
              records (except those automatically generated by Cloudflare) to be added to the
              zone.

              See
              [Multi-signer DNSSEC](https://developers.cloudflare.com/dns/dnssec/multi-signer-dnssec/)
              for details.

          dnssec_presigned: If true, allows Cloudflare to transfer in a DNSSEC-signed zone including
              signatures from an external provider, without requiring Cloudflare to sign any
              records on the fly.

              Note that this feature has some limitations. See
              [Cloudflare as Secondary](https://developers.cloudflare.com/dns/zone-setups/zone-transfers/cloudflare-as-secondary/setup/#dnssec)
              for details.

          dnssec_use_nsec3: If true, enables the use of NSEC3 together with DNSSEC on the zone. Combined
              with setting dnssec_presigned to true, this enables the use of NSEC3 records
              when transferring in from an external provider. If dnssec_presigned is instead
              set to false (default), NSEC3 records will be generated and signed at request
              time.

              See
              [DNSSEC with NSEC3](https://developers.cloudflare.com/dns/dnssec/enable-nsec3/)
              for details.

          status: Status of DNSSEC, based on user-desired state and presence of necessary records.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._patch(
            path_template("/zones/{zone_id}/dnssec", zone_id=zone_id),
            body=maybe_transform(
                {
                    "dnssec_multi_signer": dnssec_multi_signer,
                    "dnssec_presigned": dnssec_presigned,
                    "dnssec_use_nsec3": dnssec_use_nsec3,
                    "status": status,
                },
                dnssec_edit_params.DNSSECEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[DNSSEC]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[DNSSEC]], ResultWrapper[DNSSEC]),
        )

    def get(
        self,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[DNSSEC]:
        """
        Details about DNSSEC status and configuration.

        Args:
          zone_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._get(
            path_template("/zones/{zone_id}/dnssec", zone_id=zone_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[DNSSEC]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[DNSSEC]], ResultWrapper[DNSSEC]),
        )


class AsyncDNSSECResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncDNSSECResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncDNSSECResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDNSSECResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncDNSSECResourceWithStreamingResponse(self)

    async def delete(
        self,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> str:
        """
        Delete DNSSEC.

        Args:
          zone_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return await self._delete(
            path_template("/zones/{zone_id}/dnssec", zone_id=zone_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[DNSSECDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(Type[str], ResultWrapper[str]),
        )

    async def edit(
        self,
        *,
        zone_id: str,
        dnssec_multi_signer: bool | Omit = omit,
        dnssec_presigned: bool | Omit = omit,
        dnssec_use_nsec3: bool | Omit = omit,
        status: Literal["active", "disabled"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[DNSSEC]:
        """
        Enable or disable DNSSEC.

        Args:
          zone_id: Identifier.

          dnssec_multi_signer: If true, multi-signer DNSSEC is enabled on the zone, allowing multiple providers
              to serve a DNSSEC-signed zone at the same time. This is required for DNSKEY
              records (except those automatically generated by Cloudflare) to be added to the
              zone.

              See
              [Multi-signer DNSSEC](https://developers.cloudflare.com/dns/dnssec/multi-signer-dnssec/)
              for details.

          dnssec_presigned: If true, allows Cloudflare to transfer in a DNSSEC-signed zone including
              signatures from an external provider, without requiring Cloudflare to sign any
              records on the fly.

              Note that this feature has some limitations. See
              [Cloudflare as Secondary](https://developers.cloudflare.com/dns/zone-setups/zone-transfers/cloudflare-as-secondary/setup/#dnssec)
              for details.

          dnssec_use_nsec3: If true, enables the use of NSEC3 together with DNSSEC on the zone. Combined
              with setting dnssec_presigned to true, this enables the use of NSEC3 records
              when transferring in from an external provider. If dnssec_presigned is instead
              set to false (default), NSEC3 records will be generated and signed at request
              time.

              See
              [DNSSEC with NSEC3](https://developers.cloudflare.com/dns/dnssec/enable-nsec3/)
              for details.

          status: Status of DNSSEC, based on user-desired state and presence of necessary records.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return await self._patch(
            path_template("/zones/{zone_id}/dnssec", zone_id=zone_id),
            body=await async_maybe_transform(
                {
                    "dnssec_multi_signer": dnssec_multi_signer,
                    "dnssec_presigned": dnssec_presigned,
                    "dnssec_use_nsec3": dnssec_use_nsec3,
                    "status": status,
                },
                dnssec_edit_params.DNSSECEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[DNSSEC]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[DNSSEC]], ResultWrapper[DNSSEC]),
        )

    async def get(
        self,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[DNSSEC]:
        """
        Details about DNSSEC status and configuration.

        Args:
          zone_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return await self._get(
            path_template("/zones/{zone_id}/dnssec", zone_id=zone_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[DNSSEC]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[DNSSEC]], ResultWrapper[DNSSEC]),
        )


class DNSSECResourceWithRawResponse:
    def __init__(self, dnssec: DNSSECResource) -> None:
        self._dnssec = dnssec

        self.delete = to_raw_response_wrapper(
            dnssec.delete,
        )
        self.edit = to_raw_response_wrapper(
            dnssec.edit,
        )
        self.get = to_raw_response_wrapper(
            dnssec.get,
        )


class AsyncDNSSECResourceWithRawResponse:
    def __init__(self, dnssec: AsyncDNSSECResource) -> None:
        self._dnssec = dnssec

        self.delete = async_to_raw_response_wrapper(
            dnssec.delete,
        )
        self.edit = async_to_raw_response_wrapper(
            dnssec.edit,
        )
        self.get = async_to_raw_response_wrapper(
            dnssec.get,
        )


class DNSSECResourceWithStreamingResponse:
    def __init__(self, dnssec: DNSSECResource) -> None:
        self._dnssec = dnssec

        self.delete = to_streamed_response_wrapper(
            dnssec.delete,
        )
        self.edit = to_streamed_response_wrapper(
            dnssec.edit,
        )
        self.get = to_streamed_response_wrapper(
            dnssec.get,
        )


class AsyncDNSSECResourceWithStreamingResponse:
    def __init__(self, dnssec: AsyncDNSSECResource) -> None:
        self._dnssec = dnssec

        self.delete = async_to_streamed_response_wrapper(
            dnssec.delete,
        )
        self.edit = async_to_streamed_response_wrapper(
            dnssec.edit,
        )
        self.get = async_to_streamed_response_wrapper(
            dnssec.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["IPV4EndpointParam"]


class IPV4EndpointParam(TypedDict, total=False):
    enabled: bool
    """Indicate whether the IPv4 endpoint is enabled for this location."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = [
    "MeetingGetMeetingByIDResponse",
    "Data",
    "DataAIConfig",
    "DataAIConfigSummarization",
    "DataAIConfigTranscription",
    "DataRecordingConfig",
    "DataRecordingConfigAudioConfig",
    "DataRecordingConfigLiveStreamingConfig",
    "DataRecordingConfigRealtimekitBucketConfig",
    "DataRecordingConfigStorageConfig",
    "DataRecordingConfigVideoConfig",
    "DataRecordingConfigVideoConfigWatermark",
    "DataRecordingConfigVideoConfigWatermarkSize",
]


class DataAIConfigSummarization(BaseModel):
    """Summary Config"""

    summary_type: Optional[
        Literal[
            "general",
            "team_meeting",
            "sales_call",
            "client_check_in",
            "interview",
            "daily_standup",
            "one_on_one_meeting",
            "lecture",
            "code_review",
        ]
    ] = None
    """Defines the style of the summary, such as general, team meeting, or sales call."""

    text_format: Optional[Literal["plain_text", "markdown"]] = None
    """Determines the text format of the summary, such as plain text or markdown."""

    word_limit: Optional[int] = None
    """Sets the maximum number of words in the meeting summary."""


class DataAIConfigTranscription(BaseModel):
    """Transcription Configurations"""

    keywords: Optional[List[str]] = None
    """Adds specific terms to improve accurate detection during transcription."""

    language: Optional[Literal["en-US", "en-IN", "de", "hi", "sv", "ru", "pl", "el", "fr", "nl"]] = None
    """Specifies the language code for transcription to ensure accurate results."""

    profanity_filter: Optional[bool] = None
    """Control the inclusion of offensive language in transcriptions."""


class DataAIConfig(BaseModel):
    """
    The AI Config allows you to customize the behavior of meeting transcriptions and summaries
    """

    summarization: Optional[DataAIConfigSummarization] = None
    """Summary Config"""

    transcription: Optional[DataAIConfigTranscription] = None
    """Transcription Configurations"""


class DataRecordingConfigAudioConfig(BaseModel):
    """Object containing configuration regarding the audio that is being recorded."""

    channel: Optional[Literal["mono", "stereo"]] = None
    """Audio signal pathway within an audio file that carries a specific sound source."""

    codec: Optional[Literal["MP3", "AAC"]] = None
    """Codec using which the recording will be encoded.

    If VP8/VP9 is selected for videoConfig, changing audioConfig is not allowed. In
    this case, the codec in the audioConfig is automatically set to vorbis.
    """

    export_file: Optional[bool] = None
    """Controls whether to export audio file seperately"""


class DataRecordingConfigLiveStreamingConfig(BaseModel):
    rtmp_url: Optional[str] = None
    """RTMP URL to stream to"""


class DataRecordingConfigRealtimekitBucketConfig(BaseModel):
    enabled: bool
    """Controls whether recordings are uploaded to RealtimeKit's bucket.

    If set to false, `download_url`, `audio_download_url`, `download_url_expiry`
    won't be generated for a recording.
    """


class DataRecordingConfigStorageConfig(BaseModel):
    type: Literal["aws", "azure", "digitalocean", "gcs", "sftp"]
    """Type of storage media."""

    auth_method: Optional[Literal["KEY", "PASSWORD"]] = None
    """Authentication method used for "sftp" type storage medium"""

    bucket: Optional[str] = None
    """Name of the storage medium's bucket."""

    host: Optional[str] = None
    """SSH destination server host for SFTP type storage medium"""

    password: Optional[str] = None
    """
    SSH destination server password for SFTP type storage medium when auth_method is
    "PASSWORD". If auth_method is "KEY", this specifies the password for the ssh
    private key.
    """

    path: Optional[str] = None
    """Path relative to the bucket root at which the recording will be placed."""

    port: Optional[float] = None
    """SSH destination server port for SFTP type storage medium"""

    private_key: Optional[str] = None
    """
    Private key used to login to destination SSH server for SFTP type storage
    medium, when auth_method used is "KEY"
    """

    region: Optional[str] = None
    """Region of the storage medium."""

    secret: Optional[str] = None
    """Secret key of the storage medium.

    Similar to `access_key`, it is only writeable by clients, not readable.
    """

    username: Optional[str] = None
    """SSH destination server username for SFTP type storage medium"""


class DataRecordingConfigVideoConfigWatermarkSize(BaseModel):
    """Size of the watermark"""

    height: Optional[int] = None
    """Height of the watermark in px"""

    width: Optional[int] = None
    """Width of the watermark in px"""


class DataRecordingConfigVideoConfigWatermark(BaseModel):
    """Watermark to be added to the recording"""

    position: Optional[Literal["left top", "right top", "left bottom", "right bottom"]] = None
    """Position of the watermark"""

    size: Optional[DataRecordingConfigVideoConfigWatermarkSize] = None
    """Size of the watermark"""

    url: Optional[str] = None
    """URL of the watermark image"""


class DataRecordingConfigVideoConfig(BaseModel):
    codec: Optional[Literal["H264", "VP8"]] = None
    """Codec using which the recording will be encoded."""

    export_file: Optional[bool] = None
    """Controls whether to export video file seperately"""

    height: Optional[int] = None
    """Height of the recording video in pixels"""

    watermark: Optional[DataRecordingConfigVideoConfigWatermark] = None
    """Watermark to be added to the recording"""

    width: Optional[int] = None
    """Width of the recording video in pixels"""


class DataRecordingConfig(BaseModel):
    """Recording Configurations to be used for this meeting.

    This level of configs takes higher preference over App level configs on the RealtimeKit developer portal.
    """

    audio_config: Optional[DataRecordingConfigAudioConfig] = None
    """Object containing configuration regarding the audio that is being recorded."""

    file_name_prefix: Optional[str] = None
    """Adds a prefix to the beginning of the file name of the recording."""

    live_streaming_config: Optional[DataRecordingConfigLiveStreamingConfig] = None

    max_seconds: Optional[float] = None
    """
    Specifies the maximum duration for recording in seconds, ranging from a minimum
    of 60 seconds to a maximum of 24 hours.
    """

    realtimekit_bucket_config: Optional[DataRecordingConfigRealtimekitBucketConfig] = None

    storage_config: Optional[DataRecordingConfigStorageConfig] = None

    video_config: Optional[DataRecordingConfigVideoConfig] = None


class Data(BaseModel):
    """Data returned by the operation"""

    id: str
    """ID of the meeting."""

    created_at: datetime
    """Timestamp the object was created at. The time is returned in ISO format."""

    updated_at: datetime
    """Timestamp the object was updated at. The time is returned in ISO format."""

    ai_config: Optional[DataAIConfig] = None
    """
    The AI Config allows you to customize the behavior of meeting transcriptions and
    summaries
    """

    live_stream_on_start: Optional[bool] = None
    """Specifies if the meeting should start getting livestreamed on start."""

    persist_chat: Optional[bool] = None
    """Specifies if Chat within a meeting should persist for a week."""

    record_on_start: Optional[bool] = None
    """
    Specifies if the meeting should start getting recorded as soon as someone joins
    the meeting.
    """

    recording_config: Optional[DataRecordingConfig] = None
    """Recording Configurations to be used for this meeting.

    This level of configs takes higher preference over App level configs on the
    RealtimeKit developer portal.
    """

    session_keep_alive_time_in_secs: Optional[float] = None
    """
    Time in seconds, for which a session remains active, after the last participant
    has left the meeting.
    """

    status: Optional[Literal["ACTIVE", "INACTIVE"]] = None
    """Whether the meeting is `ACTIVE` or `INACTIVE`.

    Users will not be able to join an `INACTIVE` meeting.
    """

    summarize_on_end: Optional[bool] = None
    """Automatically generate summary of meetings using transcripts.

    Requires Transcriptions to be enabled, and can be retrieved via Webhooks or
    summary API.
    """

    title: Optional[str] = None
    """Title of the meeting."""


class MeetingGetMeetingByIDResponse(BaseModel):
    success: bool
    """Success status of the operation"""

    data: Optional[Data] = None
    """Data returned by the operation"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from ....._models import BaseModel

__all__ = ["Client", "Conn"]


class Conn(BaseModel):
    id: Optional[str] = None
    """UUID of the Cloudflare Tunnel connection."""

    client_id: Optional[str] = None
    """UUID of the Cloudflare Tunnel connector."""

    client_version: Optional[str] = None
    """The cloudflared version used to establish this connection."""

    colo_name: Optional[str] = None
    """The Cloudflare data center used for this connection."""

    is_pending_reconnect: Optional[bool] = None
    """
    Cloudflare continues to track connections for several minutes after they
    disconnect. This is an optimization to improve latency and reliability of
    reconnecting. If `true`, the connection has disconnected but is still being
    tracked. If `false`, the connection is actively serving traffic.
    """

    opened_at: Optional[datetime] = None
    """Timestamp of when the connection was established."""

    origin_ip: Optional[str] = None
    """The public IP address of the host running cloudflared."""

    uuid: Optional[str] = None
    """UUID of the Cloudflare Tunnel connection."""


class Client(BaseModel):
    """
    A client (typically cloudflared) that maintains connections to a Cloudflare data center.
    """

    id: Optional[str] = None
    """UUID of the Cloudflare Tunnel connection."""

    arch: Optional[str] = None
    """The cloudflared OS architecture used to establish this connection."""

    config_version: Optional[int] = None
    """The version of the remote tunnel configuration.

    Used internally to sync cloudflared with the Zero Trust dashboard.
    """

    conns: Optional[List[Conn]] = None
    """The Cloudflare Tunnel connections between your origin and Cloudflare's edge."""

    features: Optional[List[str]] = None
    """Features enabled for the Cloudflare Tunnel."""

    run_at: Optional[datetime] = None
    """Timestamp of when the tunnel connection was started."""

    version: Optional[str] = None
    """The cloudflared version used to establish this connection."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

from .geo_restrictions_param import GeoRestrictionsParam
from ..custom_hostnames.bundle_method import BundleMethod

__all__ = ["CustomCertificateEditParams"]


class CustomCertificateEditParams(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier."""

    bundle_method: BundleMethod
    """
    A ubiquitous bundle has the highest probability of being verified everywhere,
    even by clients using outdated or unusual trust stores. An optimal bundle uses
    the shortest chain and newest intermediates. And the force bundle verifies the
    chain, but does not otherwise modify it.
    """

    certificate: str
    """The zone's SSL certificate or certificate and the intermediate(s)."""

    custom_csr_id: str
    """The identifier for the Custom CSR that was used."""

    deploy: Literal["staging", "production"]
    """The environment to deploy the certificate to, defaults to production"""

    geo_restrictions: GeoRestrictionsParam
    """
    Specify the region where your private key can be held locally for optimal TLS
    performance. HTTPS connections to any excluded data center will still be fully
    encrypted, but will incur some latency while Keyless SSL is used to complete the
    handshake with the nearest allowed data center. Options allow distribution to
    only to U.S. data centers, only to E.U. data centers, or only to highest
    security data centers. Default distribution is to all Cloudflare datacenters,
    for optimal performance.
    """

    policy: str
    """
    Specify the policy that determines the region where your private key will be
    held locally. HTTPS connections to any excluded data center will still be fully
    encrypted, but will incur some latency while Keyless SSL is used to complete the
    handshake with the nearest allowed data center. Any combination of countries,
    specified by their two letter country code
    (https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2#Officially_assigned_code_elements)
    can be chosen, such as 'country: IN', as well as 'region: EU' which refers to
    the EU region. If there are too few data centers satisfying the policy, it will
    be rejected. Note: The API accepts this field as either "policy" or
    "policy_restrictions" in requests. Responses return this field as
    "policy_restrictions".
    """

    private_key: str
    """The zone's private key."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["ManagedGetParams"]


class ManagedGetParams(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier."""

    with_mapped_resource_counts: bool
    """Include `mapped_resources` for each label"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union
from typing_extensions import TypeAlias

from .lockdown_ip_configuration import LockdownIPConfiguration
from .lockdown_cidr_configuration import LockdownCIDRConfiguration

__all__ = ["Configuration", "ConfigurationItem"]

ConfigurationItem: TypeAlias = Union[LockdownIPConfiguration, LockdownCIDRConfiguration]

Configuration: TypeAlias = List[ConfigurationItem]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["PageRuleListParams"]


class PageRuleListParams(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier."""

    direction: Literal["asc", "desc"]
    """The direction used to sort returned Page Rules."""

    match: Literal["any", "all"]
    """When set to `all`, all the search requirements must match.

    When set to `any`, only one of the search requirements has to match.
    """

    order: Literal["status", "priority"]
    """The field used to sort returned Page Rules."""

    status: Literal["active", "disabled"]
    """The status of the Page Rule."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["IdentityProviderType"]

IdentityProviderType: TypeAlias = Literal[
    "onetimepin",
    "azureAD",
    "saml",
    "centrify",
    "facebook",
    "github",
    "google-apps",
    "google",
    "linkedin",
    "oidc",
    "okta",
    "onelogin",
    "pingone",
    "yandex",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .analytics import (
    AnalyticsResource,
    AsyncAnalyticsResource,
    AnalyticsResourceWithRawResponse,
    AsyncAnalyticsResourceWithRawResponse,
    AnalyticsResourceWithStreamingResponse,
    AsyncAnalyticsResourceWithStreamingResponse,
)
from .reverse_dns import (
    ReverseDNSResource,
    AsyncReverseDNSResource,
    ReverseDNSResourceWithRawResponse,
    AsyncReverseDNSResourceWithRawResponse,
    ReverseDNSResourceWithStreamingResponse,
    AsyncReverseDNSResourceWithStreamingResponse,
)
from .dns_firewall import (
    DNSFirewallResource,
    AsyncDNSFirewallResource,
    DNSFirewallResourceWithRawResponse,
    AsyncDNSFirewallResourceWithRawResponse,
    DNSFirewallResourceWithStreamingResponse,
    AsyncDNSFirewallResourceWithStreamingResponse,
)

__all__ = [
    "AnalyticsResource",
    "AsyncAnalyticsResource",
    "AnalyticsResourceWithRawResponse",
    "AsyncAnalyticsResourceWithRawResponse",
    "AnalyticsResourceWithStreamingResponse",
    "AsyncAnalyticsResourceWithStreamingResponse",
    "ReverseDNSResource",
    "AsyncReverseDNSResource",
    "ReverseDNSResourceWithRawResponse",
    "AsyncReverseDNSResourceWithRawResponse",
    "ReverseDNSResourceWithStreamingResponse",
    "AsyncReverseDNSResourceWithStreamingResponse",
    "DNSFirewallResource",
    "AsyncDNSFirewallResource",
    "DNSFirewallResourceWithRawResponse",
    "AsyncDNSFirewallResourceWithRawResponse",
    "DNSFirewallResourceWithStreamingResponse",
    "AsyncDNSFirewallResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ...._models import BaseModel

__all__ = ["TelemetryKeysResponse"]


class TelemetryKeysResponse(BaseModel):
    key: str

    last_seen_at: float = FieldInfo(alias="lastSeenAt")

    type: Literal["string", "boolean", "number"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal, Required, TypedDict

__all__ = ["RouteListParams"]


class RouteListParams(TypedDict, total=False):
    account_id: Required[str]
    """Cloudflare account ID"""

    comment: str
    """Optional remark describing the route."""

    existed_at: str
    """
    If provided, include only resources that were created (and not deleted) before
    this time. URL encoded.
    """

    is_deleted: bool
    """If `true`, only include deleted routes.

    If `false`, exclude deleted routes. If empty, all routes will be included.
    """

    network_subset: str
    """If set, only list routes that are contained within this IP range."""

    network_superset: str
    """If set, only list routes that contain this IP range."""

    page: float
    """Page number of paginated results."""

    per_page: float
    """Number of results to display."""

    route_id: str
    """UUID of the route."""

    tun_types: List[Literal["cfd_tunnel", "warp_connector", "warp", "magic", "ip_sec", "gre", "cni"]]
    """The types of tunnels to filter by, separated by commas."""

    tunnel_id: str
    """UUID of the tunnel."""

    virtual_network_id: str
    """UUID of the virtual network."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["VideoStorageUsageParams"]


class VideoStorageUsageParams(TypedDict, total=False):
    account_id: Required[str]
    """The account identifier tag."""

    creator: str
    """A user-defined identifier for the media creator."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, cast

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ...pagination import SyncV4PagePaginationArray, AsyncV4PagePaginationArray
from ..._base_client import AsyncPaginator, make_request_options
from ...types.aisearch import token_list_params, token_create_params, token_update_params
from ...types.aisearch.token_list_response import TokenListResponse
from ...types.aisearch.token_read_response import TokenReadResponse
from ...types.aisearch.token_create_response import TokenCreateResponse
from ...types.aisearch.token_update_response import TokenUpdateResponse

__all__ = ["TokensResource", "AsyncTokensResource"]


class TokensResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> TokensResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return TokensResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TokensResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return TokensResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        cf_api_id: str,
        cf_api_key: str,
        name: str,
        legacy: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TokenCreateResponse:
        """
        Create a new token.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/ai-search/tokens", account_id=account_id),
            body=maybe_transform(
                {
                    "cf_api_id": cf_api_id,
                    "cf_api_key": cf_api_key,
                    "name": name,
                    "legacy": legacy,
                },
                token_create_params.TokenCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[TokenCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[TokenCreateResponse], ResultWrapper[TokenCreateResponse]),
        )

    def update(
        self,
        id: str,
        *,
        account_id: str,
        cf_api_id: str,
        cf_api_key: str,
        name: str,
        legacy: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TokenUpdateResponse:
        """
        Update token.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._put(
            path_template("/accounts/{account_id}/ai-search/tokens/{id}", account_id=account_id, id=id),
            body=maybe_transform(
                {
                    "cf_api_id": cf_api_id,
                    "cf_api_key": cf_api_key,
                    "name": name,
                    "legacy": legacy,
                },
                token_update_params.TokenUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[TokenUpdateResponse]._unwrapper,
            ),
            cast_to=cast(Type[TokenUpdateResponse], ResultWrapper[TokenUpdateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        search: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePaginationArray[TokenListResponse]:
        """
        List tokens.

        Args:
          page: Page number (1-indexed).

          per_page: Number of results per page.

          search: Filter tokens whose name contains this string (case-insensitive).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/ai-search/tokens", account_id=account_id),
            page=SyncV4PagePaginationArray[TokenListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "per_page": per_page,
                        "search": search,
                    },
                    token_list_params.TokenListParams,
                ),
            ),
            model=TokenListResponse,
        )

    def delete(
        self,
        id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Delete token.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            path_template("/accounts/{account_id}/ai-search/tokens/{id}", account_id=account_id, id=id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[object]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )

    def read(
        self,
        id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TokenReadResponse:
        """
        Read token.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            path_template("/accounts/{account_id}/ai-search/tokens/{id}", account_id=account_id, id=id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[TokenReadResponse]._unwrapper,
            ),
            cast_to=cast(Type[TokenReadResponse], ResultWrapper[TokenReadResponse]),
        )


class AsyncTokensResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncTokensResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTokensResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTokensResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncTokensResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        cf_api_id: str,
        cf_api_key: str,
        name: str,
        legacy: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TokenCreateResponse:
        """
        Create a new token.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/ai-search/tokens", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "cf_api_id": cf_api_id,
                    "cf_api_key": cf_api_key,
                    "name": name,
                    "legacy": legacy,
                },
                token_create_params.TokenCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[TokenCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[TokenCreateResponse], ResultWrapper[TokenCreateResponse]),
        )

    async def update(
        self,
        id: str,
        *,
        account_id: str,
        cf_api_id: str,
        cf_api_key: str,
        name: str,
        legacy: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TokenUpdateResponse:
        """
        Update token.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._put(
            path_template("/accounts/{account_id}/ai-search/tokens/{id}", account_id=account_id, id=id),
            body=await async_maybe_transform(
                {
                    "cf_api_id": cf_api_id,
                    "cf_api_key": cf_api_key,
                    "name": name,
                    "legacy": legacy,
                },
                token_update_params.TokenUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[TokenUpdateResponse]._unwrapper,
            ),
            cast_to=cast(Type[TokenUpdateResponse], ResultWrapper[TokenUpdateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        search: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[TokenListResponse, AsyncV4PagePaginationArray[TokenListResponse]]:
        """
        List tokens.

        Args:
          page: Page number (1-indexed).

          per_page: Number of results per page.

          search: Filter tokens whose name contains this string (case-insensitive).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/ai-search/tokens", account_id=account_id),
            page=AsyncV4PagePaginationArray[TokenListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "per_page": per_page,
                        "search": search,
                    },
                    token_list_params.TokenListParams,
                ),
            ),
            model=TokenListResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Delete token.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            path_template("/accounts/{account_id}/ai-search/tokens/{id}", account_id=account_id, id=id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[object]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )

    async def read(
        self,
        id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TokenReadResponse:
        """
        Read token.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/ai-search/tokens/{id}", account_id=account_id, id=id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[TokenReadResponse]._unwrapper,
            ),
            cast_to=cast(Type[TokenReadResponse], ResultWrapper[TokenReadResponse]),
        )


class TokensResourceWithRawResponse:
    def __init__(self, tokens: TokensResource) -> None:
        self._tokens = tokens

        self.create = to_raw_response_wrapper(
            tokens.create,
        )
        self.update = to_raw_response_wrapper(
            tokens.update,
        )
        self.list = to_raw_response_wrapper(
            tokens.list,
        )
        self.delete = to_raw_response_wrapper(
            tokens.delete,
        )
        self.read = to_raw_response_wrapper(
            tokens.read,
        )


class AsyncTokensResourceWithRawResponse:
    def __init__(self, tokens: AsyncTokensResource) -> None:
        self._tokens = tokens

        self.create = async_to_raw_response_wrapper(
            tokens.create,
        )
        self.update = async_to_raw_response_wrapper(
            tokens.update,
        )
        self.list = async_to_raw_response_wrapper(
            tokens.list,
        )
        self.delete = async_to_raw_response_wrapper(
            tokens.delete,
        )
        self.read = async_to_raw_response_wrapper(
            tokens.read,
        )


class TokensResourceWithStreamingResponse:
    def __init__(self, tokens: TokensResource) -> None:
        self._tokens = tokens

        self.create = to_streamed_response_wrapper(
            tokens.create,
        )
        self.update = to_streamed_response_wrapper(
            tokens.update,
        )
        self.list = to_streamed_response_wrapper(
            tokens.list,
        )
        self.delete = to_streamed_response_wrapper(
            tokens.delete,
        )
        self.read = to_streamed_response_wrapper(
            tokens.read,
        )


class AsyncTokensResourceWithStreamingResponse:
    def __init__(self, tokens: AsyncTokensResource) -> None:
        self._tokens = tokens

        self.create = async_to_streamed_response_wrapper(
            tokens.create,
        )
        self.update = async_to_streamed_response_wrapper(
            tokens.update,
        )
        self.list = async_to_streamed_response_wrapper(
            tokens.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            tokens.delete,
        )
        self.read = async_to_streamed_response_wrapper(
            tokens.read,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .overrides import (
    OverridesResource,
    AsyncOverridesResource,
    OverridesResourceWithRawResponse,
    AsyncOverridesResourceWithRawResponse,
    OverridesResourceWithStreamingResponse,
    AsyncOverridesResourceWithStreamingResponse,
)
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from .packages.packages import (
    PackagesResource,
    AsyncPackagesResource,
    PackagesResourceWithRawResponse,
    AsyncPackagesResourceWithRawResponse,
    PackagesResourceWithStreamingResponse,
    AsyncPackagesResourceWithStreamingResponse,
)

__all__ = ["WAFResource", "AsyncWAFResource"]


class WAFResource(SyncAPIResource):
    @cached_property
    def overrides(self) -> OverridesResource:
        return OverridesResource(self._client)

    @cached_property
    def packages(self) -> PackagesResource:
        return PackagesResource(self._client)

    @cached_property
    def with_raw_response(self) -> WAFResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return WAFResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> WAFResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return WAFResourceWithStreamingResponse(self)


class AsyncWAFResource(AsyncAPIResource):
    @cached_property
    def overrides(self) -> AsyncOverridesResource:
        return AsyncOverridesResource(self._client)

    @cached_property
    def packages(self) -> AsyncPackagesResource:
        return AsyncPackagesResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncWAFResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncWAFResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncWAFResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncWAFResourceWithStreamingResponse(self)


class WAFResourceWithRawResponse:
    def __init__(self, waf: WAFResource) -> None:
        self._waf = waf

    @cached_property
    def overrides(self) -> OverridesResourceWithRawResponse:
        return OverridesResourceWithRawResponse(self._waf.overrides)

    @cached_property
    def packages(self) -> PackagesResourceWithRawResponse:
        return PackagesResourceWithRawResponse(self._waf.packages)


class AsyncWAFResourceWithRawResponse:
    def __init__(self, waf: AsyncWAFResource) -> None:
        self._waf = waf

    @cached_property
    def overrides(self) -> AsyncOverridesResourceWithRawResponse:
        return AsyncOverridesResourceWithRawResponse(self._waf.overrides)

    @cached_property
    def packages(self) -> AsyncPackagesResourceWithRawResponse:
        return AsyncPackagesResourceWithRawResponse(self._waf.packages)


class WAFResourceWithStreamingResponse:
    def __init__(self, waf: WAFResource) -> None:
        self._waf = waf

    @cached_property
    def overrides(self) -> OverridesResourceWithStreamingResponse:
        return OverridesResourceWithStreamingResponse(self._waf.overrides)

    @cached_property
    def packages(self) -> PackagesResourceWithStreamingResponse:
        return PackagesResourceWithStreamingResponse(self._waf.packages)


class AsyncWAFResourceWithStreamingResponse:
    def __init__(self, waf: AsyncWAFResource) -> None:
        self._waf = waf

    @cached_property
    def overrides(self) -> AsyncOverridesResourceWithStreamingResponse:
        return AsyncOverridesResourceWithStreamingResponse(self._waf.overrides)

    @cached_property
    def packages(self) -> AsyncPackagesResourceWithStreamingResponse:
        return AsyncPackagesResourceWithStreamingResponse(self._waf.packages)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from ...._models import BaseModel

__all__ = ["BGPPrefix", "BGPSignalOpts", "OnDemand"]


class BGPSignalOpts(BaseModel):
    enabled: Optional[bool] = None
    """
    Whether control of advertisement of the prefix to the Internet is enabled to be
    performed via BGP signal
    """

    modified_at: Optional[datetime] = None
    """Last time BGP signaling control was toggled.

    This field is null if BGP signaling has never been enabled.
    """


class OnDemand(BaseModel):
    advertised: Optional[bool] = None
    """Prefix advertisement status to the Internet.

    This field is only not 'null' if on demand is enabled.
    """

    advertised_modified_at: Optional[datetime] = None
    """Last time the advertisement status was changed.

    This field is only not 'null' if on demand is enabled.
    """

    on_demand_enabled: Optional[bool] = None
    """
    Whether advertisement of the prefix to the Internet may be dynamically enabled
    or disabled.
    """

    on_demand_locked: Optional[bool] = None
    """
    Whether the advertisement status of the prefix is locked, meaning it cannot be
    changed.
    """


class BGPPrefix(BaseModel):
    id: Optional[str] = None
    """Identifier of BGP Prefix."""

    asn: Optional[int] = None
    """Autonomous System Number (ASN) the prefix will be advertised under."""

    asn_prepend_count: Optional[int] = None
    """Number of times to prepend the Cloudflare ASN to the BGP AS-Path attribute"""

    auto_advertise_withdraw: Optional[bool] = None
    """
    Determines if Cloudflare advertises a BYOIP BGP prefix even when there is no
    matching BGP prefix in the Magic routing table. When true, Cloudflare will
    automatically withdraw the BGP prefix when there are no matching BGP routes, and
    will resume advertising when there is at least one matching BGP route.
    """

    bgp_signal_opts: Optional[BGPSignalOpts] = None

    cidr: Optional[str] = None
    """IP Prefix in Classless Inter-Domain Routing format."""

    created_at: Optional[datetime] = None

    modified_at: Optional[datetime] = None

    on_demand: Optional[OnDemand] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict
from typing_extensions import Literal, Required, TypedDict

__all__ = ["UploadCreateParams"]


class UploadCreateParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier."""

    base64: Required[Literal[True]]
    """Whether the file contents are base64-encoded. Must be `true`."""

    body: Required[Dict[str, str]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .trace import Trace as Trace
from .trace_item import TraceItem as TraceItem
from .trace_create_params import TraceCreateParams as TraceCreateParams
from .trace_create_response import TraceCreateResponse as TraceCreateResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, cast

import httpx

from ..._types import Body, Query, Headers, NotGiven, not_given
from ..._utils import path_template
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ..._base_client import make_request_options
from ...types.waiting_rooms.status_get_response import StatusGetResponse

__all__ = ["StatusesResource", "AsyncStatusesResource"]


class StatusesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> StatusesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return StatusesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> StatusesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return StatusesResourceWithStreamingResponse(self)

    def get(
        self,
        waiting_room_id: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> StatusGetResponse:
        """Fetches the status of a configured waiting room.

        Response fields include:

        1. `status`: String indicating the status of the waiting room. The possible
           status are:
           - **not_queueing** indicates that the configured thresholds have not been met
             and all users are going through to the origin.
           - **queueing** indicates that the thresholds have been met and some users are
             held in the waiting room.
           - **event_prequeueing** indicates that an event is active and is currently
             prequeueing users before it starts.
           - **suspended** indicates that the room is suspended.
        2. `event_id`: String of the current event's `id` if an event is active,
           otherwise an empty string.
        3. `estimated_queued_users`: Integer of the estimated number of users currently
           waiting in the queue.
        4. `estimated_total_active_users`: Integer of the estimated number of users
           currently active on the origin.
        5. `max_estimated_time_minutes`: Integer of the maximum estimated time currently
           presented to the users.

        Args:
          zone_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not waiting_room_id:
            raise ValueError(f"Expected a non-empty value for `waiting_room_id` but received {waiting_room_id!r}")
        return self._get(
            path_template(
                "/zones/{zone_id}/waiting_rooms/{waiting_room_id}/status",
                zone_id=zone_id,
                waiting_room_id=waiting_room_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[StatusGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[StatusGetResponse], ResultWrapper[StatusGetResponse]),
        )


class AsyncStatusesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncStatusesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncStatusesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncStatusesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncStatusesResourceWithStreamingResponse(self)

    async def get(
        self,
        waiting_room_id: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> StatusGetResponse:
        """Fetches the status of a configured waiting room.

        Response fields include:

        1. `status`: String indicating the status of the waiting room. The possible
           status are:
           - **not_queueing** indicates that the configured thresholds have not been met
             and all users are going through to the origin.
           - **queueing** indicates that the thresholds have been met and some users are
             held in the waiting room.
           - **event_prequeueing** indicates that an event is active and is currently
             prequeueing users before it starts.
           - **suspended** indicates that the room is suspended.
        2. `event_id`: String of the current event's `id` if an event is active,
           otherwise an empty string.
        3. `estimated_queued_users`: Integer of the estimated number of users currently
           waiting in the queue.
        4. `estimated_total_active_users`: Integer of the estimated number of users
           currently active on the origin.
        5. `max_estimated_time_minutes`: Integer of the maximum estimated time currently
           presented to the users.

        Args:
          zone_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not waiting_room_id:
            raise ValueError(f"Expected a non-empty value for `waiting_room_id` but received {waiting_room_id!r}")
        return await self._get(
            path_template(
                "/zones/{zone_id}/waiting_rooms/{waiting_room_id}/status",
                zone_id=zone_id,
                waiting_room_id=waiting_room_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[StatusGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[StatusGetResponse], ResultWrapper[StatusGetResponse]),
        )


class StatusesResourceWithRawResponse:
    def __init__(self, statuses: StatusesResource) -> None:
        self._statuses = statuses

        self.get = to_raw_response_wrapper(
            statuses.get,
        )


class AsyncStatusesResourceWithRawResponse:
    def __init__(self, statuses: AsyncStatusesResource) -> None:
        self._statuses = statuses

        self.get = async_to_raw_response_wrapper(
            statuses.get,
        )


class StatusesResourceWithStreamingResponse:
    def __init__(self, statuses: StatusesResource) -> None:
        self._statuses = statuses

        self.get = to_streamed_response_wrapper(
            statuses.get,
        )


class AsyncStatusesResourceWithStreamingResponse:
    def __init__(self, statuses: AsyncStatusesResource) -> None:
        self._statuses = statuses

        self.get = async_to_streamed_response_wrapper(
            statuses.get,
        )
